#include <winver.h>

#define QMMP_FILEVERSION 2,3,1,0
#define QMMP_PRODUCTVERSION "2.3.1.0"

IDI_ICON1		ICON		DISCARDABLE	"images\\ico\\qmmp.ico"
IDI_ICON2		ICON		DISCARDABLE	"images\\ico\\qmmp_file.ico"

VS_VERSION_INFO VERSIONINFO
 FILEVERSION QMMP_FILEVERSION
 PRODUCTVERSION QMMP_FILEVERSION
 FILEFLAGS 0x0L
 FILEFLAGSMASK 0x3fL
 FILEOS 0x00040004L
 FILETYPE 0x1L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "000004b0"
        BEGIN
            VALUE "CompanyName", "Qmmp Development Team"
            VALUE "FileDescription", "Qt-based Multimedia Player"
            VALUE "FileVersion", QMMP_PRODUCTVERSION
            VALUE "LegalCopyright", "Copyright (C) 2006-2026 Qmmp Development Team"
            VALUE "InternalName", "qmmp"
            VALUE "OriginalFilename", "qmmp.exe"
            VALUE "ProductName", "Qmmp"
            VALUE "ProductVersion", QMMP_PRODUCTVERSION
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x0, 1200
    END
END
