/***************************************************************************
 *   Copyright (C) 2016-2025 by Ilya Kotov                                 *
 *   forkotov02@ya.ru                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
 ***************************************************************************/

#include <QRegularExpression>
#include <QMessageBox>
#include <archive.h>
#include <archive_entry.h>
#include <taglib/taglib.h>
#include "decoder_archive.h"
#include "archivetagreader.h"
#include "archiveinputdevice.h"
#include "decoderarchivefactory.h"

// DecoderArchiveFactory
bool DecoderArchiveFactory::canDecode(QIODevice *) const
{
    return false;
}

DecoderProperties DecoderArchiveFactory::properties() const
{
    DecoderProperties properties;
    properties.name = tr("Archive Plugin");
    properties.filters = QStringList { u"*.rar"_s, u"*.zip"_s };
    properties.description = tr("Archives");
    properties.contentTypes = QStringList { u"application/zip"_s, u"application/x-rar-compressed"_s };
    properties.shortName = "archive"_L1;
    properties.hasAbout = true;
    properties.hasSettings = false;
    properties.noInput = true;
    properties.protocols = QStringList { u"rar"_s, u"zip"_s };
    return properties;
}

Decoder *DecoderArchiveFactory::create(const QString &url, QIODevice *)
{
    return new DecoderArchive(url);
}

QList<TrackInfo *> DecoderArchiveFactory::createPlayList(const QString &path, TrackInfo::Parts parts, QStringList *)
{
    QList<TrackInfo *> list;
    struct archive_entry *entry = nullptr;

    struct archive *a = archive_read_new();
    archive_read_support_filter_all(a);
    archive_read_support_format_all(a);

    QString archivePath, requiredFilePath;
    if(path.contains(u"://"_s))
    {
        requiredFilePath = path.section(QLatin1Char('#'), -1);
        archivePath = path;
        archivePath.remove(QRegularExpression(u"^.+://"_s));
        archivePath.remove(QRegularExpression(u"#.+$"_s));
    }
    else
    {
        archivePath = path;
    }

    if(archive_read_open_filename(a, archivePath.toLocal8Bit().constData(), 10240) != ARCHIVE_OK)
    {
        qCWarning(plugin, "DecoderArchiveFactory: unable to open archive; libarchive error: %s", archive_error_string(a));
        return list;
    }

    while (archive_read_next_header(a, &entry) == ARCHIVE_OK)
    {
        if(archive_read_has_encrypted_entries(a) == 1) //skip encrypted archives
        {
            qDeleteAll(list);
            list.clear();
            archive_read_free(a);
            return list;
        }

        if(archive_entry_filetype(entry) == AE_IFREG)
        {
            QString filePath = QString::fromLocal8Bit(archive_entry_pathname(entry));
            if(!filePath.startsWith(QLatin1Char('/')))
                filePath.prepend(QLatin1Char('/'));

            if(!requiredFilePath.isEmpty() && filePath != requiredFilePath)
            {
                archive_read_data_skip(a);
                continue;
            }

            //is this file supported by qmmp?
            QList<DecoderFactory *> filtered;
            for(DecoderFactory *f : Decoder::findByFileExtension(filePath))
            {
                if(!f->properties().noInput)
                    filtered << f; //add all factories with streaming input
            }

            if(!filtered.isEmpty())
            {
                list << new TrackInfo(QStringLiteral("%1://%2#%3").arg(archivePath.section(QLatin1Char('.'), -1).toLower(), archivePath, filePath));

                ArchiveInputDevice dev(a, entry, nullptr);
                ArchiveTagReader reader(&dev, list.last()->path());

                if(!dev.isOpen())
                {
                    qDeleteAll(list);
                    list.clear();
                    archive_read_free(a);
                    return list;
                }

                if(parts & TrackInfo::MetaData)
                    list.last()->setValues(reader.metaData());

                TagLib::AudioProperties *ap = reader.audioProperties();

                if((parts & TrackInfo::Properties) && ap)
                {
                    list.last()->setValue(Qmmp::BITRATE, ap->bitrate());
                    list.last()->setValue(Qmmp::SAMPLERATE, ap->sampleRate());
                    list.last()->setValue(Qmmp::CHANNELS, ap->channels());
                    list.last()->setValue(Qmmp::FILE_SIZE, dev.size());
                }

                if(ap)
                    list.last()->setDuration(reader.audioProperties()->lengthInMilliseconds());
            }
        }
        archive_read_data_skip(a);
    }
    archive_read_free(a);

    return list;
}

MetaDataModel* DecoderArchiveFactory::createMetaDataModel(const QString &path, bool readOnly)
{
    Q_UNUSED(path);
    Q_UNUSED(readOnly);
    return nullptr;
}

QDialog *DecoderArchiveFactory::createSettings(QWidget *)
{
    return nullptr;
}

void DecoderArchiveFactory::showAbout(QWidget *parent)
{
    QMessageBox::about(parent, tr("About Archive Reader Plugin"),
                       tr("Qmmp Archive Reader Plugin") + QChar::LineFeed +
                       tr("Compiled against:") + QChar::LineFeed +
                       QStringLiteral("libarchive-") + QStringLiteral(ARCHIVE_VERSION_ONLY_STRING) + QChar::LineFeed +
                       QStringLiteral("TagLib-%1.%2.%3\n").arg(TAGLIB_MAJOR_VERSION).arg(TAGLIB_MINOR_VERSION).arg(TAGLIB_PATCH_VERSION) +
                       tr("Written by: Ilya Kotov <forkotov02@ya.ru>"));
}

QString DecoderArchiveFactory::translation() const
{
    return QLatin1String(":/archive_plugin_");
}
