/***************************************************************************
 *   Copyright (C) 2006-2025 by Ilya Kotov                                 *
 *   forkotov02@ya.ru                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
 ***************************************************************************/
#include "skin.h"
#include "skinnedplaystatus.h"

SkinnedPlayStatus::SkinnedPlayStatus(QWidget *parent) : PixmapWidget(parent)
{
   setState(Qmmp::Stopped);
}

void SkinnedPlayStatus::setState(Qmmp::State state)
{
   m_state = state;
   switch(state)
   {
   case Qmmp::Playing:
       setPixmap(skin()->getItem(Skin::PLAY));
       break;
   case Qmmp::Paused:
       setPixmap(skin()->getItem(Skin::PAUSE));
       break;
   default:
       setPixmap(skin()->getItem(Skin::STOP));
   }
}

void SkinnedPlayStatus::updateSkin()
{
    setState(m_state);
}
